"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.constructConfigPath = constructConfigPath;
exports.filterFiles = filterFiles;
exports.getFiles = getFiles;
exports.getFilesFromBranchDiff = getFilesFromBranchDiff;
exports.getLintReportPath = getLintReportPath;
exports.getRootGitPath = getRootGitPath;
exports.getStagedFiles_CI = getStagedFiles_CI;
exports.getStagedFiles_preCommit = getStagedFiles_preCommit;
exports.resolvePath = resolvePath;
var _path = _interopRequireDefault(require("path"));
var _child_process = require("child_process");
var _logger = require("../../utils/logger");
var _fileUtils = require("../../utils/fileUtils");
/**
 * @function getFilesFromBranchDiff  - filter file path From git diff json 
 * @param {JSON} diffJson -  Git Diff JSON  (This JSON get by git diff api)
 * @returns {Array<String>} 
 */

function getFilesFromBranchDiff(diffJson) {
  const rootPath = getRootGitPath();
  const diffFiles = diffJson === null || diffJson === void 0 ? void 0 : diffJson.diffs.map(diff => {
    return _path.default.resolve(rootPath, diff === null || diff === void 0 ? void 0 : diff.new_path);
  });
  return filterFiles(diffFiles);
}

/**
 * @function getLintReportPath - construct lint report path
 * @returns {string}
 */

function getLintReportPath() {
  const {
    lintReportPath
  } = (0, _fileUtils.getLintConfiguration)();
  return lintReportPath;
}

/**
 * @function resolvePath - resolve given  file path 
 * @param {string} resolveFilePath -  file path 
 * @returns {string}
 */

function resolvePath(resolveFilePath) {
  const file_Path = _path.default.resolve(resolveFilePath);
  if (!(0, _fileUtils.checkIfFileExists)(file_Path)) {
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, "file does not exist ..");
    process.exit(0);
  }
  return file_Path;
}
function getStagedFiles_preCommit() {
  const command = "git diff --cached -U0 HEAD --name-only";
  const child = (0, _child_process.spawnSync)(command, {
    shell: true
  });
  const diffFiles = child.stdout.toString().split("\n").map(filePath => _path.default.resolve(getRootGitPath(), filePath));
  diffFiles.pop(); // remove last empty string
  return diffFiles;
}

/**
 * @function getStagedFiles_CI - check Linter trigger from and Filter lint files
 * @param {JSON} branch_diff_json - Git Diff JSON  (This JSON get by git diff api)
 * @returns {Array<String>}
 */

function getStagedFiles_CI(branch_diff_json) {
  if (typeof branch_diff_json === "boolean" && !branch_diff_json) {
    _logger.Logger.log(_logger.Logger.FAILURE_TYPE, `Expected Either branch diff json or files. but received ${typeof branch_diff_json === "boolean" ? `empty string` : "empty diff"} `);
    process.exit(0);
  }
  return getFilesFromBranchDiff(branch_diff_json);
}

/**
 * @function getRootGitPath -  get git root path
 * @returns {string}
 */

function getRootGitPath() {
  const childProcess = (0, _child_process.spawnSync)("git", ["rev-parse", "--show-toplevel"], {
    shell: true
  });
  return childProcess.stdout.toString().split("\n").filter(Boolean).pop();
}

/**
 * 
 * @function getFiles - method returns a js and feature and branchDiff based on env
 * @param {string} env - gives running env
 * @param {string} dir - path of directory or file
 * @param {path} branchDiffPath - path of branch diff json
 * @returns {{JsFiles:[],featureFiles:[],branch_diff_json:[]}}
 */
function getFiles(env, dir, branchDiffPath = false) {
  if (env == 'dev') {
    const listOfFiles = dir && (0, _fileUtils.checkIfFileExists)(dir) ? dir : false;
    return (0, _fileUtils.getFilesPathFromRootDir)(listOfFiles, []);
  }
  if (env == 'ci') {
    const _branch_diff_json = branchDiffPath && (0, _fileUtils.checkIfFileExists)(branchDiffPath) ? require(resolvePath(branchDiffPath)) : [{}];
    return {
      ...getStagedFiles_CI(_branch_diff_json),
      branch_diff_json: _branch_diff_json
    };
  }
}
function constructConfigPath() {
  return _path.default.resolve(process.cwd(), "lint.config.js");
}

/**
 * @function filterFiles - filter javascript files. omit feature files
 * @param {Array<String>} lintFiles - linter files as Array
 * @returns {Array<String>} 
 */

function filterFiles(lintFiles) {
  return lintFiles.reduce((files, currentFile) => {
    if (currentFile.includes('.feature')) {
      files.featureFiles.push(currentFile);
    } else {
      files.JsFiles.push(currentFile);
    }
    return files;
  }, {
    featureFiles: [],
    JsFiles: []
  });
}